#include <assert.h>

#include "common.h"
#include "keys.h"
#include "hmac_sha1.h"
#include "rsa.h"
#include "utils.h"

enum _license_features 
{
   feature1 = 0x01,
   feature2 = 0x02,
   feature3 = 0x04,
   feature4 = 0x08,
   feature5 = 0x10,
   feature6 = 0x20,
   expires_30days = 0x40,
   expires_1year  = 0x80,
   expires_never  = 0xC0,
};

#define RSA_PRIVKEY "privkey"
#define RSA_PUBKEY "pubkey"


static int genl_symm_generate(char *license,
                              size_t license_size,
                              u64 target_id,
                              u16 expiration_date,
                              u16 features,
                              const void *symm_key);

static int genl_asymm_generate(char *license,
                               size_t license_size,
                               char *partial_sig,
                               size_t part_sig_size,
                               u64 target_id,
                               u8 lic_features,
                               RSA *rsa_key);
int main(void) 
{
    char userlicstr[100];
    u64 target_id = 0x1122334455667788;
    {
        u16 expiration_date = 0xaabb;
        u16 lic_features = feature1 | feature3;
        unsigned char symm_key[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 
                                     1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };

        genl_symm_generate(userlicstr, sizeof(userlicstr),
                           target_id, expiration_date, lic_features,
                           symm_key);

        printf("hmac license = ");
        utils_print_lic(stdout, userlicstr);
        printf("\n");
    }

    {
        RSA *rsa_key = NULL;
        char partial_signature[256];

        /* read the key */
        keys_file2rsakey(&rsa_key, RSA_PRIVKEY, KEYS_PRIVKEY);

        genl_asymm_generate(userlicstr, sizeof(userlicstr),
                            partial_signature, sizeof(partial_signature),
                            target_id, feature1 | feature3 | expires_1year,
                            rsa_key);

        printf("\nrsa license  = ");
        utils_print_lic(stdout, userlicstr);
        printf("\n");

        printf("partial signature to be added to the application =\n%s\n",
               partial_signature); 
    }

    return 0;
}


static int genl_symm_generate(char *license,
                              size_t license_size,
                              u64 target_id,
                              u16 expiration_date,
                              u16 features,
                              const void *symm_key)
{
    unsigned char licdata[12];
    unsigned char mac[SHA1_OUTPUT_LEN];
    unsigned char userlic[10];
    assert(license_size > sizeof(licdata)*2);

    /* copy data into place */
    memcpy(licdata,
           (char *)&target_id, sizeof(target_id));
    memcpy(licdata + sizeof(target_id),
           (char *)&expiration_date, sizeof(expiration_date));
    memcpy(licdata + sizeof(target_id) + sizeof(expiration_date),
           (char *)&features, sizeof(features));

    hmac_sha1(mac, sizeof(mac),
              licdata, sizeof(licdata),
              symm_key, sizeof(symm_key));

    /* create the license string */
    memcpy(userlic,
           (char *)&expiration_date, sizeof(expiration_date));
    memcpy(userlic + sizeof(expiration_date),
           (char *)&features, sizeof(features));
    memcpy(userlic + sizeof(expiration_date) + sizeof(features),
           mac, sizeof(userlic) - (sizeof(expiration_date) + sizeof(features)));

    /* format it nicely */
    utils_bin2str(license, license_size, userlic, sizeof(userlic));

    return 0;
}


static int genl_asymm_generate(char *license,
                               size_t license_size,
                               char *partial_sig,
                               size_t part_sig_size,                                
                               u64 target_id,
                               u8 lic_features,
                               RSA *rsa_key)
{
    unsigned char licdata[9];
    unsigned char signature[128];
    size_t sig_len;
    unsigned char userlic[10];

    /* copy data into place */
    memcpy(licdata, 
           (char *)&target_id, sizeof(target_id)); 
    memcpy(licdata + sizeof(target_id),
           (char *)&lic_features, sizeof(lic_features));

    /* sign the license */
    rsa_openssl_sign(signature, &sig_len,
                     licdata, sizeof(licdata),
                     rsa_key);

    /* create the license string */
    memcpy(userlic, 
           (char *)&lic_features, sizeof(lic_features));
    memcpy(userlic + sizeof(lic_features),
           signature, sizeof(userlic) - sizeof(lic_features));
    utils_bin2str(license, license_size, userlic, sizeof(userlic));

    /* create the part that should be shipped with the sw */
    utils_bin2str(partial_sig, part_sig_size, 
                  signature + sizeof(userlic) - sizeof(lic_features),
                  sig_len - sizeof(userlic) + sizeof(lic_features));

    /* free the key */
    keys_free_rsa_key(rsa_key);

    return 0;
}

