#include <assert.h>

#include "common.h"
#include "utils.h"

int utils_pad(void *dst,
              size_t dst_size, 
              const void *src,
              size_t src_len, 
              unsigned char pad,
              size_t total_len)
{
    int i;

    assert(total_len <= dst_size);
    assert(src_len <= total_len);
    assert(dst);
    assert(src || (!src && src_len == 0));

    if (src_len > 0) {
        memcpy(dst, src, src_len);
    }
    for (i=src_len; i<total_len; i++) {
        *((char *)dst+i) = pad;
    }
    return 0;   
}


int utils_xor(void *dst,
              size_t dst_size,
              const void *x1,
              const void *x2)
{
    int i;

    assert(dst && x1 && x2);

    for (i=0; i < dst_size; i++) {
        *((char *)dst+i) = *((const char *)x1+i) ^ *((const char *)x2+i);
    }
    return 0;
}


void utils_append(void *dst,
                  size_t dst_size,
                  const void *x1,
                  size_t x1_len,
                  const void *x2,
                  size_t x2_len)
{
    assert(dst && x1 && x2);
    assert(dst_size >= x1_len + x2_len);

    memcpy(dst, x1, x1_len);
    memcpy((char*)dst + x1_len, x2, x2_len); 
}


void utils_dumphex(const void *src, 
                   size_t src_len)
{
   int i;

   assert(src);

   for (i=0; i<src_len; i++) {
       printf("0x%02X ", (unsigned char)*((const char *)src+i));
   }  
   printf("\n");
}


static unsigned int hex2bin(char c)
{
    if(c>='0' && c<='9')
        return c-'0';
    if(c>='a' && c<='f')
        return c-'a'+10;
    if(c>='A' && c<='F')
        return c-'A'+10;
    return 0xffff;
}


int utils_binstr2bin(unsigned char *bin,
                     size_t bin_size,
                     const char *binstr,
                     size_t binstr_length)
{
   int i;

   assert(bin && binstr);

   if (binstr_length > strlen(binstr))
       return -1;   
   if (bin_size < strlen(binstr)/2)
       return -1;
   if (strlen(binstr) % 2 != 0)
       return -1;

   for (i = 0; i < binstr_length/2; i++) {
       int v = hex2bin(binstr[i*2])*16 + hex2bin(binstr[i*2+1]);
       if (v <= 255) {
           bin[i] = v;
       } else {
           return -1;
       }       
   }

   return 0;
} 

int utils_bstrenc2bin(unsigned char *bin,
                      size_t bin_size,
                      size_t *bin_length,
                      const char *binstr,
                      unsigned int index)
{
    int len;
    int _index = 0;
    int pos = 0;
    int ret = -1;

    assert(bin && binstr && bin_length);

    do {
        len = hex2bin(binstr[pos])*16 + hex2bin(binstr[pos+1]);
        if (len > 255) break;
        
        if (index == _index) {
            if (strlen(binstr) < pos+2+len*2) break;
            ret = utils_binstr2bin(bin, bin_size,
                                   binstr+pos+2, len*2);
            *bin_length = len;
            break;
        } 
        else {
            _index++;
            pos += 2+len*2;
        }
    } while (pos < strlen(binstr));

    return ret;
}

int utils_bstrenc2str(char *str,
                      size_t str_size,
                      size_t *str_length,
                      const char *binstr,
                      unsigned int index)
{
    int len;
    int _index = 0;
    int pos = 0;
    int ret = -1;

    assert(str && binstr && str_length);

    do {
        len = hex2bin(binstr[pos])*16 + hex2bin(binstr[pos+1]);
        if (len > 255) break;

        if (index == _index) {
            if (strlen(binstr) < pos+2+len*2) break;
            memcpy(str, binstr+pos+2, len*2);
            *str_length = len*2;
            str[*str_length] = '\0';
            ret = 0;
            break;
        }
        else {
            _index++;
            pos += 2+len*2;
        }
    } while (pos < strlen(binstr));

    return ret;
}


int utils_bin2str(unsigned char *str,
                  size_t str_size,
                  const unsigned char *bin,
                  size_t bin_len)
{
    int i;

    assert(str && bin);
    assert(str_size >= 2*bin_len+1);

    for (i=0; i<bin_len; i++) {
        u8 hibits = (bin[i] & 0xF0) >> 4;
        u8 lobits = bin[i] & 0x0F;
        str[i*2] = hibits + 'A';
        str[i*2+1] = lobits + 'A';
    }   
    str[i*2] = '\0';

    return 0;
}


int utils_str2bin(unsigned char *bin,
                  size_t bin_size,
                  size_t *bin_len,
                  const char *str)
{
    int i;

    assert(str && bin);
    assert(bin_size >= strlen(str)/2);
    assert(strlen(str) % 2 == 0);

    for (i=0; i<strlen(str)/2; i++) {
        bin[i] = (str[i*2]-'A')*16 + (str[i*2+1]-'A');
    }
    *bin_len = i;

    return 0;
}


int utils_print_lic(FILE *f,
                    const unsigned char *str)
{
    int i;

    assert(str);

    for (i=0; i<strlen(str); i++) {
        fprintf(f, "%c", str[i]);
        if (((i +1) % 5 == 0) && (i < strlen(str)-1)) {
            fprintf(f, "-");
        }
    }

    return 0;
}
