#include <assert.h>

#include "common.h"
#include "keys.h"
#include "hmac_sha1.h"
#include "rsa.h"
#include "utils.h"

enum _license_features 
{
   feature1 = 0x01,
   feature2 = 0x02,
   feature3 = 0x04,
   feature4 = 0x08,
   feature5 = 0x10,
   feature6 = 0x20,
   expires_30days = 0x40,
   expires_1year  = 0x80,
   expires_never  = 0xC0,
};

#define RSA_PRIVKEY "privkey"
#define RSA_PUBKEY "pubkey"

static int verl_symm_verify(const char *license,
                            u64 target_id,
                            const void *symm_key,
                            u16 *expiration_date,
                            u16 *features,
                            u8 *isValid);

static int verl_asymm_verify(const char* license,
                           const char* partial_sig,
                           u64 target_id,
                           RSA *rsa_key,
                           u8 *lic_features,
                           u8 *isValid);

int main(void)
{
    u64 target_id = 0x1122334455667788;
    u8 isValid;

    {
        char userlicstr[] = "LLKKAFAAABEBIPIOIALC";
        unsigned char symm_key[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
                                     1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };
        u16 expiration_date;
        u16 features;

        verl_symm_verify(userlicstr, target_id, symm_key,
                         &expiration_date, 
                         &features,
                         &isValid);
        fprintf(stdout, "The hmac license is %s\n", isValid ? "VALID" : "INVALID");
    }

    {
        RSA *rsa_key = NULL;
        char userlicstr[] = "IFCMBJEKPCLHNPPCFDGD";
        unsigned char partial_sig[] = "COHJGAFLHAICFKIKBOHNEFIKMKNBLEEEOLIBOMLMFJJKJAGCGNIODOLMDKINBNKFLNADAPLOCIKJNDOHOOHBJAGAFAKPLFJJKOACOBDIKNNLMJBABGJFIMMKAEAMKPMAPDAJAMOCBLKBKEIFPAEIMLGDDNDNLCIAJOBBMMLCHDKGEGDKBAGCDMAMNKIPBGMABOAMOIEBKKMHELEPLEJJNALMEELMAFIPIKGDCBMJMEFJDK";
        u8 lic_features;

        /* read the key */
        keys_file2rsakey(&rsa_key, RSA_PUBKEY, KEYS_PUBKEY);

        verl_asymm_verify(userlicstr, partial_sig, target_id, rsa_key,
                          &lic_features,
                          &isValid);

        fprintf(stdout, "The rsa license is %s\n", isValid ? "VALID" : "INVALID");
    }

    return 0;
}


static int verl_symm_verify(const char *license,
                            u64 target_id,
                            const void *symm_key,
                            u16 *expiration_date,
                            u16 *features,
                            u8 *isValid)
{
    unsigned char lic_bin[10];
    size_t licdata_len;
    unsigned char licdata[12];
    unsigned char mac[SHA1_OUTPUT_LEN];
    unsigned char userlic[10];

    /* parse the license */
    utils_str2bin(lic_bin, sizeof(lic_bin), &licdata_len, license); 
    memcpy(expiration_date, lic_bin, sizeof(*expiration_date));
    memcpy(features, lic_bin+sizeof(*expiration_date), sizeof(*features));

    /* reconstruct the HMACed data */
    memcpy(licdata,
           (char *)&target_id, sizeof(target_id));
    memcpy(licdata + sizeof(target_id),
           (char *)expiration_date, sizeof(*expiration_date));
    memcpy(licdata + sizeof(target_id) + sizeof(*expiration_date),
           (char *)features, sizeof(*features));

    hmac_sha1(mac, sizeof(mac),
              licdata, sizeof(licdata),
              symm_key, sizeof(symm_key));

    {
        /* compare the left most bits */
        int offset = sizeof(*expiration_date)+sizeof(*features);
        *isValid = 0;
        if (memcmp(lic_bin+offset, mac, sizeof(lic_bin) - offset) == 0) {
            *isValid = 1;
        } 
    }

    return 0;
}


static int verl_asymm_verify(const char* license,
                           const char* partial_sig,
                           u64 target_id,
                           RSA *rsa_key,
                           u8 *lic_features,
                           u8 *isValid)
{
    unsigned char lic_bin[10];
    size_t licdata_len;
    unsigned char licdata[9];
    unsigned char signature[512];
    unsigned char bin_signature[128];
    size_t sig_len;

    /* parse the license */
    utils_str2bin(lic_bin, sizeof(lic_bin), &licdata_len, license);
    memcpy(lic_features, lic_bin, sizeof(*lic_features));

    /* reconstruct the signed data */
    memcpy(licdata,
           (char *)&target_id, sizeof(target_id));
    memcpy(licdata + sizeof(target_id),
           (char *)lic_features, sizeof(*lic_features));

    /* reconstruct the signature */
    strlcpy(signature, license + sizeof(*lic_features)*2, sizeof(signature));
    strlcat(signature, partial_sig, sizeof(signature));
    utils_str2bin(bin_signature, sizeof(bin_signature), &sig_len, signature);

    /* try to verify the license */
    rsa_openssl_verify(bin_signature, sig_len,
                       licdata, sizeof(licdata),
                       rsa_key,
                       isValid);

    /* free the key */
    keys_free_rsa_key(rsa_key);

    return 0;
}

