// factory.h
// Generic abstract factory template.
// Copyright 2001, 2003 Jim Hyslop.
//
// This file may be freely used, modified and distributed, provided that
// the accompanying copyright notice remains intact.
//
// $Id: factory.h,v 1.18 2003/04/02 03:45:41 Jim Exp $
//
// The generic abstract factory template is an implementation of the
// Abstract Class Factory pattern, as a template (see "Design Patterns:
// Elements of Reusable Object-Oriented Software", E. Gamma, R. Helm,
// R. Johnson, J. Vlissides, Addison Wesley [1995] )
//
// To use the template, you need to provide a base class and (optionally)
// a key class. The base class must provide:
//  - a unique identifier
//
// The key class must be able to be used as a key in a std::map, i.e. it must
//   - implement copy and assignment semantics
//   - provide bool operator< () const; (strictly speaking, 
//     it must provide bool less<classIDKey>() )
// Default is std::string.
//
// Steps to using the factory:
// 1 - Create the base class and its derivatives
// 2 - Register each class in the factory by instantiating a 
//     registerInFactory<> template class - do this in one file only (the
//     class implementation file is the perfect place for this)
// 3 - create the object by calling create() and passing it the same
//     value used when you instantiated the registerInFactory object.
// For example:
//   base header:
//   class Base { /* whatever (don't forget the virtual dtor! */ };
//
//   base implementation:
//   registerInFactory<Base, Base, std::string> registerBase("Base");
//
//   derived header:
//   class Derived : public Base { /* whatever */ };
//
//   derived implementation:
//   registerInFactory<Base, Derived, std::string> registerDer("Derived");
//
//   code that instantiates the classes:
//   std::auto_ptr<Base> newBase = GenericFactory<Base>::instance().create("Base");
//   std::auto_ptr<Base> newDerived = GenericFactory<Base>::instance().create("Derived");
//
// New derivatives can be added without affecting the existing code.

#ifndef FACTORY_HEADER_DEFINED
#define FACTORY_HEADER_DEFINED
#include <map>
#include <memory>
#include <vector>
#include <string>

template < typename T, typename Storage>
class PersistentStorage
{
public:
    explicit PersistentStorage( Storage & s );
    T & restore();
    void save( const T & ) const;
};

typedef std::string defaultIDKeyType;

class DoNothingClass
{
};

template 
<
    class AncestorType, 
    typename classIDKey=defaultIDKeyType,
    typename Source = DoNothingClass,
    typename Sink = DoNothingClass,
    typename Parameter = DoNothingClass,
    typename ConcreteCreateFn = std::auto_ptr<AncestorType> (*)( ),
    typename ConcreteSaveFn = void (*)( Sink &, const Parameter & )
>
class GenericFactory
{
    // Singleton implementation - private ctor & copying, with
    // no implementation on the copying.
	GenericFactory(const GenericFactory&); // Not implemented
	GenericFactory &operator=(const GenericFactory&); // Not implemented
    GenericFactory() {}

    struct Functions
    {
        ConcreteCreateFn creator;
        ConcreteSaveFn saver;
    };

    // FN_REGISTRY is the registry of all the ConcreteCreateFn
    // pointers registered.  Functions are registered using the
    // registerFunctions member function (see below).
    typedef std::map<classIDKey, Functions> FN_REGISTRY;
    FN_REGISTRY registry;

public:

    // Singleton access.
    static GenericFactory &instance()
    {
        static GenericFactory theInstance;
        return theInstance;
    }

    // Classes derived from AncestorType call this function once
    // per program to register the class ID key, and a pointer to
    // the function that creates the class.
    void registerFunctions(const classIDKey & clName, ConcreteCreateFn creator, ConcreteSaveFn saver = NULL)
    {
        Functions funcs;
        funcs.creator = creator;
        funcs.saver = saver;
        registry[clName]=funcs;
    }

    // Create a new class of the type specified by className.
    std::auto_ptr<AncestorType> create(const classIDKey &className) const
    {
        std::auto_ptr<AncestorType> ret(0);
        typename FN_REGISTRY::const_iterator regEntry=registry.find(className);
        if (regEntry != registry.end()) {
            std::auto_ptr<AncestorType> tmp((*regEntry).second.creator());
            ret=tmp;
        }
        return ret;
    }

    // Create a new instance of the specified class, using the Source
    // to provide information for the non-default constructor.
    std::auto_ptr<AncestorType> create( const classIDKey &className, Source & source ) const
    {
        std::auto_ptr<AncestorType> ret(0);
        typename FN_REGISTRY::const_iterator regEntry=registry.find(className);
        if (regEntry != registry.end()) {
            std::auto_ptr<AncestorType> tmp((*regEntry).second.creator( source ));
            ret=tmp;
        }
        return ret;
    }

    void save( const classIDKey &className, Sink &sink, Parameter & param ) const
    {
        typename FN_REGISTRY::const_iterator regEntry=registry.find(className);
        if (regEntry != registry.end()) {
            (*regEntry).second.saver( sink, param );
        }
    }

    // This function is used by the driver program to get a list of
    // classes that are registered.
    std::vector<classIDKey> getRegisteredClasses() const
    {
        std::vector<classIDKey> ret(registry.size());
        int count;
        typename FN_REGISTRY::const_iterator regEntry;
        for (count=0,
             regEntry=registry.begin();
             regEntry!=registry.end();
             ++regEntry, ++count)
        {
            ret[count]=(*regEntry).first;
        }
        return ret;
    }
};

template <class AncestorType, class ManufacturedType, typename classIDKey=defaultIDKeyType>
class registerInFactory
{
public:
    static std::auto_ptr<AncestorType> createInstance()
    {
        return std::auto_ptr<AncestorType>(new ManufacturedType);
    }
    registerInFactory(const classIDKey &id)
    {
        GenericFactory<AncestorType, classIDKey>::instance().registerFunctions(id, createInstance);
    }
};

template
<
    class AncestorType,
    typename AncestorParameter,
    class ConcreteType,
    typename ConcreteParameter,
    typename Source,
    typename Sink,
    typename ConcreteCreateFn = std::auto_ptr<AncestorType> (*)( Source & ),
    typename classIDKey = defaultIDKeyType
>
class RegisterInFactoryWithParams
{
public:
     
    static std::auto_ptr< AncestorType > createInstance( Source & source )
    {
        return std::auto_ptr< AncestorType >(
            new ConcreteType( PersistentStorage< ConcreteParameter, Source >( source ).restore() )
            );
    }
    static void saveInstance( Sink & sink, const AncestorParameter & ip )
    {
        PersistentStorage< ConcreteParameter, Sink >( sink ).save( 
                static_cast< const ConcreteParameter & >( ip ) );
    }

    RegisterInFactoryWithParams( const classIDKey &id )
    {
        GenericFactory
        <
            AncestorType, 
            classIDKey, 
            Source, 
            Sink,
            AncestorParameter, 
            ConcreteCreateFn
        >
        ::instance().registerFunctions( id, createInstance, saveInstance );
    }
};

#endif
